import java.io.*;
import java.util.*;

public class DemoIO
{
    public static void main(String[] args) throws IOException {
        List<Employee> employees = new ArrayList(List.of(
            new Employee(2000, "Karel Omacka"),
            new Employee(2001, "Josef Novak")
        ));
        
        backupTxt(new File("out.txt"), employees);
        //backupTxt(System.out, employees);
        employees.clear();
        employees.addAll(recoverTxt("out.txt"));
        
        backupBin("out.bin", employees);
        
        writeTxtToBin(new FileOutputStream("out.txt"),employees);
        writeTxtToBin(System.out, employees);
        
    }
    
    public static void backupTxt(File file, List<Employee> emp) throws IOException {
        BufferedWriter writer = new BufferedWriter(new FileWriter(file));
        try {
            for (Employee e: emp) {
                //writer.write(e.getId());
                //writer.write(" ");
                //writer.write(e.getName());
                writer.write(e.toString());
                writer.newLine();
            }
        } finally {
            writer.close();
        }
    }
    
    public static void backupTxtTWR(File file, List<Employee> emp) throws IOException {
        try(FileWriter writer = new FileWriter(file)) {
            backupTxtTWR(writer, emp);
            //for (Employee e: emp) {
            //    writer.write(e.toString());
            //    writer.newLine();
            //}
        }
    }
    
    public static void backupTxtTWR(Writer wr, List<Employee> emp) throws IOException {
        BufferedWriter writer = new BufferedWriter(wr);
        for (Employee e: emp) {
            writer.write(e.toString());
            writer.newLine();
        }
        writer.flush();
    }
    
    public static void backupTxt2(File file, List<Employee> emp) {
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(file));
            for (Employee e: emp) {
                //writer.write(e.getId());
                //writer.write(" ");
                //writer.write(e.getName());
                writer.write(e.toString());
                writer.newLine();
            }
        } catch(IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                if (writer != null) {
                    writer.close();
                }
            } catch(IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public static List<Employee> recoverTxt(String file) throws IOException {
        try (FileReader reader = new FileReader(file)) {
            return recoverTxt(reader);
        }
    }
    
    public static List<Employee> recoverTxt(Reader rd) throws IOException {
        List<Employee> list = new ArrayList<>();
        
        BufferedReader reader = new BufferedReader(rd);
        try {
            String line = reader.readLine();
            while (line != null) {
                String[] rec = line.split(" ", 2);
                if (rec.length != 2) {
                    throw new IOException("aaa");
                }
                int id = Integer.parseInt(rec[0]);
                
                list.add(new Employee(id, rec[1]));
                
                line = reader.readLine();
            }
        } catch(NumberFormatException ex) {
            throw new IOException(ex);
        }
        return list;
    }
    
    public static void backupBin(String file, List<Employee> emp) throws IOException {
        try (ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(file))) {
            for (Employee e: emp) {
                os.writeInt(e.getId());
                os.writeObject(e.getName());
            }
        }
    }
    
    public static List<Employee> recoverBin(String file) throws IOException {
        List<Employee> list = new ArrayList<>();
        ObjectInputStream is = new ObjectInputStream(new FileInputStream(file));
        
        
        //is.readInt();
        //(String) is.readObject();
        
        return list;
    }
    
    public static void writeTxtToBin(OutputStream os, List<Employee> emp) throws IOException {
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(os, "WIN-1250"));
        for (Employee e: emp) {
            writer.write(e.toString());
            writer.newLine();
        }
        writer.flush();
    }
    
    public static void systemInput() throws IOException {
         InputStreamReader isr = new InputStreamReader(System.in);
         int c = isr.read();
         while (c != -1) {
             System.out.println((char)c);
             c = isr.read();
         }
    }
}







